import pytest
from google.protobuf.json_format import MessageToDict

from devolo_plc_api.device_api.devolo_idl_proto_deviceapi_ledsettings_pb2 import LedSettingsGet, LedSettingsSetResponse
from devolo_plc_api.device_api.devolo_idl_proto_deviceapi_updatefirmware_pb2 import UpdateFirmwareCheck, UpdateFirmwareStart
from devolo_plc_api.device_api.devolo_idl_proto_deviceapi_wifinetwork_pb2 import (WifiConnectedStationsGet,
                                                                                  WifiGuestAccessGet,
                                                                                  WifiGuestAccessSetResponse,
                                                                                  WifiNeighborAPsGet,
                                                                                  WifiRepeatedAPsGet,
                                                                                  WifiWpsPbcStart)
from devolo_plc_api.exceptions.feature import FeatureNotSupported


class TestDeviceApi:

    @pytest.mark.parametrize("feature", ["[]"])
    def test_unsupported_feature(self, device_api):
        with pytest.raises(FeatureNotSupported):
            device_api.get_led_setting()

    @pytest.mark.parametrize("feature", [""])
    def test_feature(self, device_api):
        assert device_api.features == ['reset', 'update', 'led', 'intmtg']

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["led"])
    async def test_async_get_led_setting(self, device_api, httpx_mock):
        led_setting_get = LedSettingsGet()
        httpx_mock.add_response(data=led_setting_get.SerializeToString())
        led_setting = await device_api.async_get_led_setting()
        assert led_setting == MessageToDict(led_setting_get,
                                            including_default_value_fields=True,
                                            preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["led"])
    def test_get_led_setting(self, device_api, httpx_mock):
        led_setting_get = LedSettingsGet()
        httpx_mock.add_response(data=led_setting_get.SerializeToString())
        led_setting = device_api.get_led_setting()
        assert led_setting == MessageToDict(led_setting_get,
                                            including_default_value_fields=True,
                                            preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["led"])
    async def test_async_set_led_setting(self, device_api, httpx_mock):
        led_setting_set = LedSettingsSetResponse()
        httpx_mock.add_response(data=led_setting_set.SerializeToString())
        assert await device_api.async_set_led_setting(True)

    @pytest.mark.parametrize("feature", ["led"])
    def test_set_led_setting(self, device_api, httpx_mock):
        led_setting_set = LedSettingsSetResponse()
        httpx_mock.add_response(data=led_setting_set.SerializeToString())
        assert device_api.set_led_setting(True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["update"])
    async def test_async_check_firmware_available(self, device_api, httpx_mock):
        firmware_available = UpdateFirmwareCheck()
        httpx_mock.add_response(data=firmware_available.SerializeToString())
        firmware = await device_api.async_check_firmware_available()
        assert firmware == MessageToDict(firmware_available,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["update"])
    def test_check_firmware_available(self, device_api, httpx_mock):
        firmware_available = UpdateFirmwareCheck()
        httpx_mock.add_response(data=firmware_available.SerializeToString())
        firmware = device_api.check_firmware_available()
        assert firmware == MessageToDict(firmware_available,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["update"])
    async def test_async_start_firmware_update(self, device_api, httpx_mock):
        firmware_update = UpdateFirmwareStart()
        httpx_mock.add_response(data=firmware_update.SerializeToString())
        assert await device_api.async_start_firmware_update()

    @pytest.mark.parametrize("feature", ["update"])
    def test_start_firmware_update(self, device_api, httpx_mock):
        firmware_update = UpdateFirmwareStart()
        httpx_mock.add_response(data=firmware_update.SerializeToString())
        assert device_api.start_firmware_update()

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_connected_station(self, device_api, httpx_mock):
        wifi_connected_stations_get = WifiConnectedStationsGet()
        httpx_mock.add_response(data=wifi_connected_stations_get.SerializeToString())
        connected_stations = await device_api.async_get_wifi_connected_station()
        assert connected_stations == MessageToDict(wifi_connected_stations_get,
                                                   including_default_value_fields=True,
                                                   preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_connected_station(self, device_api, httpx_mock):
        wifi_connected_stations_get = WifiConnectedStationsGet()
        httpx_mock.add_response(data=wifi_connected_stations_get.SerializeToString())
        connected_stations = device_api.get_wifi_connected_station()
        assert connected_stations == MessageToDict(wifi_connected_stations_get,
                                                   including_default_value_fields=True,
                                                   preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_guest_access(self, device_api, httpx_mock):
        wifi_guest_access_get = WifiGuestAccessGet()
        httpx_mock.add_response(data=wifi_guest_access_get.SerializeToString())
        wifi_guest_access = await device_api.async_get_wifi_guest_access()
        assert wifi_guest_access == MessageToDict(wifi_guest_access_get,
                                                  including_default_value_fields=True,
                                                  preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_guest_access(self, device_api, httpx_mock):
        wifi_guest_access_get = WifiGuestAccessGet()
        httpx_mock.add_response(data=wifi_guest_access_get.SerializeToString())
        wifi_guest_access = device_api.get_wifi_guest_access()
        assert wifi_guest_access == MessageToDict(wifi_guest_access_get,
                                                  including_default_value_fields=True,
                                                  preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_set_wifi_guest_access(self, device_api, httpx_mock):
        wifi_guest_access_set = WifiGuestAccessSetResponse()
        httpx_mock.add_response(data=wifi_guest_access_set.SerializeToString())
        assert await device_api.async_set_wifi_guest_access(True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_set_wifi_guest_access(self, device_api, httpx_mock):
        wifi_guest_access_set = WifiGuestAccessSetResponse()
        httpx_mock.add_response(data=wifi_guest_access_set.SerializeToString())
        assert device_api.set_wifi_guest_access(True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_neighbor_access_points(self, device_api, httpx_mock):
        wifi_neighbor_accesspoints_get = WifiNeighborAPsGet()
        httpx_mock.add_response(data=wifi_neighbor_accesspoints_get.SerializeToString())
        wifi_neighbor_access_points = await device_api.async_get_wifi_neighbor_access_points()
        assert wifi_neighbor_access_points == MessageToDict(wifi_neighbor_accesspoints_get,
                                                            including_default_value_fields=True,
                                                            preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_neighbor_access_points(self, device_api, httpx_mock):
        wifi_neighbor_accesspoints_get = WifiNeighborAPsGet()
        httpx_mock.add_response(data=wifi_neighbor_accesspoints_get.SerializeToString())
        wifi_neighbor_access_points = device_api.get_wifi_neighbor_access_points()
        assert wifi_neighbor_access_points == MessageToDict(wifi_neighbor_accesspoints_get,
                                                            including_default_value_fields=True,
                                                            preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_repeated_access_points(self, device_api, httpx_mock):
        wifi_repeated_accesspoints_get = WifiRepeatedAPsGet()
        httpx_mock.add_response(data=wifi_repeated_accesspoints_get.SerializeToString())
        wifi_repeated_access_points = await device_api.async_get_wifi_repeated_access_points()
        assert wifi_repeated_access_points == MessageToDict(wifi_repeated_accesspoints_get,
                                                            including_default_value_fields=True,
                                                            preserving_proto_field_name=True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_repeated_access_points(self, device_api, httpx_mock):
        wifi_repeated_accesspoints_get = WifiRepeatedAPsGet()
        httpx_mock.add_response(data=wifi_repeated_accesspoints_get.SerializeToString())
        wifi_repeated_access_points = device_api.get_wifi_repeated_access_points()
        assert wifi_repeated_access_points == MessageToDict(wifi_repeated_accesspoints_get,
                                                            including_default_value_fields=True,
                                                            preserving_proto_field_name=True)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_start_wps(self, device_api, httpx_mock):
        wps = WifiWpsPbcStart()
        httpx_mock.add_response(data=wps.SerializeToString())
        assert await device_api.async_start_wps()

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_start_wps(self, device_api, httpx_mock):
        wps = WifiWpsPbcStart()
        httpx_mock.add_response(data=wps.SerializeToString())
        assert device_api.start_wps()



import json
import pathlib
import socket

from zeroconf import ServiceInfo

file = pathlib.Path(__file__).parent / ".." / "test_data.json"
with file.open("r") as fh:
    test_data = json.load(fh)


class MockZeroconf:

    def get_service_info(self, service_type, name):
        service_info = ServiceInfo(service_type, name)
        service_info.addresses = [socket.inet_aton(test_data['ip'])]
        service_info.text = b"\x09new=value"
        return service_info


class MockServiceBrowser:

    def __init__(self, zc, st, sc, addr=None):
        sc[0]()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: devolo_idl_proto_plcnetapi_setuserdevicename.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='devolo_idl_proto_plcnetapi_setuserdevicename.proto',
  package='plcnet.api',
  syntax='proto3',
  serialized_pb=_b('\n2devolo_idl_proto_plcnetapi_setuserdevicename.proto\x12\nplcnet.api\"B\n\x11SetUserDeviceName\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x18\n\x10user_device_name\x18\x02 \x01(\t\"\xe2\x01\n\x19SetUserDeviceNameResponse\x12<\n\x06result\x18\x01 \x01(\x0e\x32,.plcnet.api.SetUserDeviceNameResponse.Result\"\x86\x01\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x13\n\x0fMACADDR_INVALID\x10\x01\x12\x13\n\x0fMACADDR_UNKNOWN\x10\x02\x12\x17\n\x13\x44\x45VICE_NAME_INVALID\x10\x03\x12\x18\n\x13\x43OMMUNICATION_ERROR\x10\xfe\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x0e\n\x06plcnetB\x04nameb\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_SETUSERDEVICENAMERESPONSE_RESULT = _descriptor.EnumDescriptor(
  name='Result',
  full_name='plcnet.api.SetUserDeviceNameResponse.Result',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MACADDR_INVALID', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MACADDR_UNKNOWN', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_NAME_INVALID', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COMMUNICATION_ERROR', index=4, number=254,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=5, number=255,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=227,
  serialized_end=361,
)
_sym_db.RegisterEnumDescriptor(_SETUSERDEVICENAMERESPONSE_RESULT)


_SETUSERDEVICENAME = _descriptor.Descriptor(
  name='SetUserDeviceName',
  full_name='plcnet.api.SetUserDeviceName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='plcnet.api.SetUserDeviceName.mac_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_device_name', full_name='plcnet.api.SetUserDeviceName.user_device_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=132,
)


_SETUSERDEVICENAMERESPONSE = _descriptor.Descriptor(
  name='SetUserDeviceNameResponse',
  full_name='plcnet.api.SetUserDeviceNameResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='plcnet.api.SetUserDeviceNameResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SETUSERDEVICENAMERESPONSE_RESULT,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=361,
)

_SETUSERDEVICENAMERESPONSE.fields_by_name['result'].enum_type = _SETUSERDEVICENAMERESPONSE_RESULT
_SETUSERDEVICENAMERESPONSE_RESULT.containing_type = _SETUSERDEVICENAMERESPONSE
DESCRIPTOR.message_types_by_name['SetUserDeviceName'] = _SETUSERDEVICENAME
DESCRIPTOR.message_types_by_name['SetUserDeviceNameResponse'] = _SETUSERDEVICENAMERESPONSE

SetUserDeviceName = _reflection.GeneratedProtocolMessageType('SetUserDeviceName', (_message.Message,), dict(
  DESCRIPTOR = _SETUSERDEVICENAME,
  __module__ = 'devolo_idl_proto_plcnetapi_setuserdevicename_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.SetUserDeviceName)
  ))
_sym_db.RegisterMessage(SetUserDeviceName)

SetUserDeviceNameResponse = _reflection.GeneratedProtocolMessageType('SetUserDeviceNameResponse', (_message.Message,), dict(
  DESCRIPTOR = _SETUSERDEVICENAMERESPONSE,
  __module__ = 'devolo_idl_proto_plcnetapi_setuserdevicename_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.SetUserDeviceNameResponse)
  ))
_sym_db.RegisterMessage(SetUserDeviceNameResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\006plcnetB\004name'))
# @@protoc_insertion_point(module_scope)



import asyncio
import time
from typing import Dict

from zeroconf import ServiceBrowser, ServiceStateChange, Zeroconf

from ..device import Device

_devices: Dict[str,
               Device] = {}


async def async_discover_network() -> Dict[str, Device]:
    """
    Discover all devices that expose the devolo device API via mDNS asynchronous.

    :return: Devices accessible via serial number.
    """
    browser = ServiceBrowser(Zeroconf(), "_dvl-deviceapi._tcp.local.", [_add])
    await asyncio.sleep(3)
    browser.cancel()
    return _devices


def discover_network() -> Dict[str, Device]:
    """
    Discover devices that expose the devolo device API via mDNS synchronous.

    :return: Devices accessible via serial number.
    """
    browser = ServiceBrowser(Zeroconf(), "_dvl-deviceapi._tcp.local.", [_add])
    time.sleep(3)
    browser.cancel()
    return _devices


def _add(zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange):
    """" Create a device object to each matching device. """
    if state_change is not ServiceStateChange.Added:
        return

    service_info = zeroconf.get_service_info(service_type, name)
    if service_info is None:
        return

    info = Device.info_from_service(service_info)
    if info is None or info["properties"]["MT"] in ("2600", "2601"):
        return  # Don't react on devolo Home Control central units

    _devices[info["properties"]["SN"]] = Device(ip=info["address"], deviceapi=info, zeroconf_instance=zeroconf)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: devolo_idl_proto_deviceapi_wifinetwork.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='devolo_idl_proto_deviceapi_wifinetwork.proto',
  package='device.api',
  syntax='proto3',
  serialized_pb=_b('\n,devolo_idl_proto_deviceapi_wifinetwork.proto\x12\ndevice.api\".\n\x11WifiParametersSet\x12\x0c\n\x04ssid\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\"C\n\x19WifiParametersSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"6\n\x12WifiGuestAccessSet\x12\x0e\n\x06\x65nable\x18\x01 \x01(\x08\x12\x10\n\x08\x64uration\x18\x02 \x01(\r\"D\n\x1aWifiGuestAccessSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"\\\n\x12WifiGuestAccessGet\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x1a\n\x12remaining_duration\x18\x02 \x01(\r\x12\x0c\n\x04ssid\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\"\xe9\x01\n\x12WifiNeighborAPsGet\x12\x43\n\x0cneighbor_aps\x18\x01 \x03(\x0b\x32-.device.api.WifiNeighborAPsGet.NeighborAPInfo\x1a\x8d\x01\n\x0eNeighborAPInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x0c\n\x04ssid\x18\x02 \x01(\t\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07\x63hannel\x18\x04 \x01(\r\x12\x0e\n\x06signal\x18\x05 \x01(\x05\x12\x13\n\x0bsignal_bars\x18\x06 \x01(\x05\"\xf7\x01\n\x12WifiRepeatedAPsGet\x12\x43\n\x0crepeated_aps\x18\x01 \x03(\x0b\x32-.device.api.WifiRepeatedAPsGet.RepeatedAPInfo\x1a\x9b\x01\n\x0eRepeatedAPInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x0c\n\x04ssid\x18\x02 \x01(\t\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07\x63hannel\x18\x04 \x01(\r\x12\x0c\n\x04rate\x18\x05 \x01(\r\x12\x0e\n\x06signal\x18\x06 \x01(\x05\x12\x13\n\x0bsignal_bars\x18\x07 \x01(\x05\"\x90\x02\n\x18WifiConnectedStationsGet\x12U\n\x12\x63onnected_stations\x18\x01 \x03(\x0b\x32\x39.device.api.WifiConnectedStationsGet.ConnectedStationInfo\x1a\x9c\x01\n\x14\x43onnectedStationInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12)\n\x08vap_type\x18\x02 \x01(\x0e\x32\x17.device.api.WifiVAPType\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07rx_rate\x18\x04 \x01(\r\x12\x0f\n\x07tx_rate\x18\x05 \x01(\r\"u\n\x19WifiRepeaterParametersSet\x12\x0c\n\x04ssid\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\x11\n\tcrossband\x18\x03 \x01(\x08\x12*\n\x0cprimary_band\x18\x04 \x01(\x0e\x32\x14.device.api.WifiBand\"K\n!WifiRepeaterParametersSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"9\n\x0fWifiWpsPbcStart\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"F\n\x1cWifiRepeaterWpsClonePbcStart\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult*z\n\nWifiResult\x12\x10\n\x0cWIFI_SUCCESS\x10\x00\x12\x15\n\x11WIFI_INVALID_SSID\x10\x01\x12\x14\n\x10WIFI_INVALID_KEY\x10\x02\x12\x14\n\x10WIFI_IS_DISABLED\x10\x03\x12\x17\n\x12WIFI_UNKNOWN_ERROR\x10\xff\x01*E\n\x08WifiBand\x12\x15\n\x11WIFI_BAND_UNKNOWN\x10\x00\x12\x10\n\x0cWIFI_BAND_2G\x10\x01\x12\x10\n\x0cWIFI_BAND_5G\x10\x02*f\n\x0bWifiVAPType\x12\x14\n\x10WIFI_VAP_UNKNOWN\x10\x00\x12\x14\n\x10WIFI_VAP_MAIN_AP\x10\x01\x12\x15\n\x11WIFI_VAP_GUEST_AP\x10\x02\x12\x14\n\x10WIFI_VAP_STATION\x10\x03\x42\x15\n\x06\x64\x65viceB\x0bwifinetworkb\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_WIFIRESULT = _descriptor.EnumDescriptor(
  name='WifiResult',
  full_name='device.api.WifiResult',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WIFI_SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_INVALID_SSID', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_INVALID_KEY', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_IS_DISABLED', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_UNKNOWN_ERROR', index=4, number=255,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1485,
  serialized_end=1607,
)
_sym_db.RegisterEnumDescriptor(_WIFIRESULT)

WifiResult = enum_type_wrapper.EnumTypeWrapper(_WIFIRESULT)
_WIFIBAND = _descriptor.EnumDescriptor(
  name='WifiBand',
  full_name='device.api.WifiBand',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WIFI_BAND_UNKNOWN', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_BAND_2G', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_BAND_5G', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1609,
  serialized_end=1678,
)
_sym_db.RegisterEnumDescriptor(_WIFIBAND)

WifiBand = enum_type_wrapper.EnumTypeWrapper(_WIFIBAND)
_WIFIVAPTYPE = _descriptor.EnumDescriptor(
  name='WifiVAPType',
  full_name='device.api.WifiVAPType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WIFI_VAP_UNKNOWN', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_VAP_MAIN_AP', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_VAP_GUEST_AP', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WIFI_VAP_STATION', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1680,
  serialized_end=1782,
)
_sym_db.RegisterEnumDescriptor(_WIFIVAPTYPE)

WifiVAPType = enum_type_wrapper.EnumTypeWrapper(_WIFIVAPTYPE)
WIFI_SUCCESS = 0
WIFI_INVALID_SSID = 1
WIFI_INVALID_KEY = 2
WIFI_IS_DISABLED = 3
WIFI_UNKNOWN_ERROR = 255
WIFI_BAND_UNKNOWN = 0
WIFI_BAND_2G = 1
WIFI_BAND_5G = 2
WIFI_VAP_UNKNOWN = 0
WIFI_VAP_MAIN_AP = 1
WIFI_VAP_GUEST_AP = 2
WIFI_VAP_STATION = 3



_WIFIPARAMETERSSET = _descriptor.Descriptor(
  name='WifiParametersSet',
  full_name='device.api.WifiParametersSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='device.api.WifiParametersSet.ssid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='device.api.WifiParametersSet.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=106,
)


_WIFIPARAMETERSSETRESPONSE = _descriptor.Descriptor(
  name='WifiParametersSetResponse',
  full_name='device.api.WifiParametersSetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.WifiParametersSetResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=108,
  serialized_end=175,
)


_WIFIGUESTACCESSSET = _descriptor.Descriptor(
  name='WifiGuestAccessSet',
  full_name='device.api.WifiGuestAccessSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enable', full_name='device.api.WifiGuestAccessSet.enable', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='duration', full_name='device.api.WifiGuestAccessSet.duration', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=177,
  serialized_end=231,
)


_WIFIGUESTACCESSSETRESPONSE = _descriptor.Descriptor(
  name='WifiGuestAccessSetResponse',
  full_name='device.api.WifiGuestAccessSetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.WifiGuestAccessSetResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=301,
)


_WIFIGUESTACCESSGET = _descriptor.Descriptor(
  name='WifiGuestAccessGet',
  full_name='device.api.WifiGuestAccessGet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enabled', full_name='device.api.WifiGuestAccessGet.enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='remaining_duration', full_name='device.api.WifiGuestAccessGet.remaining_duration', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ssid', full_name='device.api.WifiGuestAccessGet.ssid', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='device.api.WifiGuestAccessGet.key', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=303,
  serialized_end=395,
)


_WIFINEIGHBORAPSGET_NEIGHBORAPINFO = _descriptor.Descriptor(
  name='NeighborAPInfo',
  full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.mac_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ssid', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.ssid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='band', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.band', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='channel', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.channel', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signal', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.signal', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signal_bars', full_name='device.api.WifiNeighborAPsGet.NeighborAPInfo.signal_bars', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=490,
  serialized_end=631,
)

_WIFINEIGHBORAPSGET = _descriptor.Descriptor(
  name='WifiNeighborAPsGet',
  full_name='device.api.WifiNeighborAPsGet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='neighbor_aps', full_name='device.api.WifiNeighborAPsGet.neighbor_aps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_WIFINEIGHBORAPSGET_NEIGHBORAPINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=398,
  serialized_end=631,
)


_WIFIREPEATEDAPSGET_REPEATEDAPINFO = _descriptor.Descriptor(
  name='RepeatedAPInfo',
  full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.mac_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ssid', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.ssid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='band', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.band', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='channel', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.channel', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rate', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.rate', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signal', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.signal', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signal_bars', full_name='device.api.WifiRepeatedAPsGet.RepeatedAPInfo.signal_bars', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=726,
  serialized_end=881,
)

_WIFIREPEATEDAPSGET = _descriptor.Descriptor(
  name='WifiRepeatedAPsGet',
  full_name='device.api.WifiRepeatedAPsGet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='repeated_aps', full_name='device.api.WifiRepeatedAPsGet.repeated_aps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_WIFIREPEATEDAPSGET_REPEATEDAPINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=634,
  serialized_end=881,
)


_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO = _descriptor.Descriptor(
  name='ConnectedStationInfo',
  full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo.mac_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vap_type', full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo.vap_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='band', full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo.band', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rx_rate', full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo.rx_rate', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tx_rate', full_name='device.api.WifiConnectedStationsGet.ConnectedStationInfo.tx_rate', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1000,
  serialized_end=1156,
)

_WIFICONNECTEDSTATIONSGET = _descriptor.Descriptor(
  name='WifiConnectedStationsGet',
  full_name='device.api.WifiConnectedStationsGet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='connected_stations', full_name='device.api.WifiConnectedStationsGet.connected_stations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=884,
  serialized_end=1156,
)


_WIFIREPEATERPARAMETERSSET = _descriptor.Descriptor(
  name='WifiRepeaterParametersSet',
  full_name='device.api.WifiRepeaterParametersSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='device.api.WifiRepeaterParametersSet.ssid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='device.api.WifiRepeaterParametersSet.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crossband', full_name='device.api.WifiRepeaterParametersSet.crossband', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='primary_band', full_name='device.api.WifiRepeaterParametersSet.primary_band', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1158,
  serialized_end=1275,
)


_WIFIREPEATERPARAMETERSSETRESPONSE = _descriptor.Descriptor(
  name='WifiRepeaterParametersSetResponse',
  full_name='device.api.WifiRepeaterParametersSetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.WifiRepeaterParametersSetResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1277,
  serialized_end=1352,
)


_WIFIWPSPBCSTART = _descriptor.Descriptor(
  name='WifiWpsPbcStart',
  full_name='device.api.WifiWpsPbcStart',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.WifiWpsPbcStart.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1354,
  serialized_end=1411,
)


_WIFIREPEATERWPSCLONEPBCSTART = _descriptor.Descriptor(
  name='WifiRepeaterWpsClonePbcStart',
  full_name='device.api.WifiRepeaterWpsClonePbcStart',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.WifiRepeaterWpsClonePbcStart.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1413,
  serialized_end=1483,
)

_WIFIPARAMETERSSETRESPONSE.fields_by_name['result'].enum_type = _WIFIRESULT
_WIFIGUESTACCESSSETRESPONSE.fields_by_name['result'].enum_type = _WIFIRESULT
_WIFINEIGHBORAPSGET_NEIGHBORAPINFO.fields_by_name['band'].enum_type = _WIFIBAND
_WIFINEIGHBORAPSGET_NEIGHBORAPINFO.containing_type = _WIFINEIGHBORAPSGET
_WIFINEIGHBORAPSGET.fields_by_name['neighbor_aps'].message_type = _WIFINEIGHBORAPSGET_NEIGHBORAPINFO
_WIFIREPEATEDAPSGET_REPEATEDAPINFO.fields_by_name['band'].enum_type = _WIFIBAND
_WIFIREPEATEDAPSGET_REPEATEDAPINFO.containing_type = _WIFIREPEATEDAPSGET
_WIFIREPEATEDAPSGET.fields_by_name['repeated_aps'].message_type = _WIFIREPEATEDAPSGET_REPEATEDAPINFO
_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO.fields_by_name['vap_type'].enum_type = _WIFIVAPTYPE
_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO.fields_by_name['band'].enum_type = _WIFIBAND
_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO.containing_type = _WIFICONNECTEDSTATIONSGET
_WIFICONNECTEDSTATIONSGET.fields_by_name['connected_stations'].message_type = _WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO
_WIFIREPEATERPARAMETERSSET.fields_by_name['primary_band'].enum_type = _WIFIBAND
_WIFIREPEATERPARAMETERSSETRESPONSE.fields_by_name['result'].enum_type = _WIFIRESULT
_WIFIWPSPBCSTART.fields_by_name['result'].enum_type = _WIFIRESULT
_WIFIREPEATERWPSCLONEPBCSTART.fields_by_name['result'].enum_type = _WIFIRESULT
DESCRIPTOR.message_types_by_name['WifiParametersSet'] = _WIFIPARAMETERSSET
DESCRIPTOR.message_types_by_name['WifiParametersSetResponse'] = _WIFIPARAMETERSSETRESPONSE
DESCRIPTOR.message_types_by_name['WifiGuestAccessSet'] = _WIFIGUESTACCESSSET
DESCRIPTOR.message_types_by_name['WifiGuestAccessSetResponse'] = _WIFIGUESTACCESSSETRESPONSE
DESCRIPTOR.message_types_by_name['WifiGuestAccessGet'] = _WIFIGUESTACCESSGET
DESCRIPTOR.message_types_by_name['WifiNeighborAPsGet'] = _WIFINEIGHBORAPSGET
DESCRIPTOR.message_types_by_name['WifiRepeatedAPsGet'] = _WIFIREPEATEDAPSGET
DESCRIPTOR.message_types_by_name['WifiConnectedStationsGet'] = _WIFICONNECTEDSTATIONSGET
DESCRIPTOR.message_types_by_name['WifiRepeaterParametersSet'] = _WIFIREPEATERPARAMETERSSET
DESCRIPTOR.message_types_by_name['WifiRepeaterParametersSetResponse'] = _WIFIREPEATERPARAMETERSSETRESPONSE
DESCRIPTOR.message_types_by_name['WifiWpsPbcStart'] = _WIFIWPSPBCSTART
DESCRIPTOR.message_types_by_name['WifiRepeaterWpsClonePbcStart'] = _WIFIREPEATERWPSCLONEPBCSTART
DESCRIPTOR.enum_types_by_name['WifiResult'] = _WIFIRESULT
DESCRIPTOR.enum_types_by_name['WifiBand'] = _WIFIBAND
DESCRIPTOR.enum_types_by_name['WifiVAPType'] = _WIFIVAPTYPE

WifiParametersSet = _reflection.GeneratedProtocolMessageType('WifiParametersSet', (_message.Message,), dict(
  DESCRIPTOR = _WIFIPARAMETERSSET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiParametersSet)
  ))
_sym_db.RegisterMessage(WifiParametersSet)

WifiParametersSetResponse = _reflection.GeneratedProtocolMessageType('WifiParametersSetResponse', (_message.Message,), dict(
  DESCRIPTOR = _WIFIPARAMETERSSETRESPONSE,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiParametersSetResponse)
  ))
_sym_db.RegisterMessage(WifiParametersSetResponse)

WifiGuestAccessSet = _reflection.GeneratedProtocolMessageType('WifiGuestAccessSet', (_message.Message,), dict(
  DESCRIPTOR = _WIFIGUESTACCESSSET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessSet)
  ))
_sym_db.RegisterMessage(WifiGuestAccessSet)

WifiGuestAccessSetResponse = _reflection.GeneratedProtocolMessageType('WifiGuestAccessSetResponse', (_message.Message,), dict(
  DESCRIPTOR = _WIFIGUESTACCESSSETRESPONSE,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessSetResponse)
  ))
_sym_db.RegisterMessage(WifiGuestAccessSetResponse)

WifiGuestAccessGet = _reflection.GeneratedProtocolMessageType('WifiGuestAccessGet', (_message.Message,), dict(
  DESCRIPTOR = _WIFIGUESTACCESSGET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessGet)
  ))
_sym_db.RegisterMessage(WifiGuestAccessGet)

WifiNeighborAPsGet = _reflection.GeneratedProtocolMessageType('WifiNeighborAPsGet', (_message.Message,), dict(

  NeighborAPInfo = _reflection.GeneratedProtocolMessageType('NeighborAPInfo', (_message.Message,), dict(
    DESCRIPTOR = _WIFINEIGHBORAPSGET_NEIGHBORAPINFO,
    __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiNeighborAPsGet.NeighborAPInfo)
    ))
  ,
  DESCRIPTOR = _WIFINEIGHBORAPSGET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiNeighborAPsGet)
  ))
_sym_db.RegisterMessage(WifiNeighborAPsGet)
_sym_db.RegisterMessage(WifiNeighborAPsGet.NeighborAPInfo)

WifiRepeatedAPsGet = _reflection.GeneratedProtocolMessageType('WifiRepeatedAPsGet', (_message.Message,), dict(

  RepeatedAPInfo = _reflection.GeneratedProtocolMessageType('RepeatedAPInfo', (_message.Message,), dict(
    DESCRIPTOR = _WIFIREPEATEDAPSGET_REPEATEDAPINFO,
    __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiRepeatedAPsGet.RepeatedAPInfo)
    ))
  ,
  DESCRIPTOR = _WIFIREPEATEDAPSGET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeatedAPsGet)
  ))
_sym_db.RegisterMessage(WifiRepeatedAPsGet)
_sym_db.RegisterMessage(WifiRepeatedAPsGet.RepeatedAPInfo)

WifiConnectedStationsGet = _reflection.GeneratedProtocolMessageType('WifiConnectedStationsGet', (_message.Message,), dict(

  ConnectedStationInfo = _reflection.GeneratedProtocolMessageType('ConnectedStationInfo', (_message.Message,), dict(
    DESCRIPTOR = _WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO,
    __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiConnectedStationsGet.ConnectedStationInfo)
    ))
  ,
  DESCRIPTOR = _WIFICONNECTEDSTATIONSGET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiConnectedStationsGet)
  ))
_sym_db.RegisterMessage(WifiConnectedStationsGet)
_sym_db.RegisterMessage(WifiConnectedStationsGet.ConnectedStationInfo)

WifiRepeaterParametersSet = _reflection.GeneratedProtocolMessageType('WifiRepeaterParametersSet', (_message.Message,), dict(
  DESCRIPTOR = _WIFIREPEATERPARAMETERSSET,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterParametersSet)
  ))
_sym_db.RegisterMessage(WifiRepeaterParametersSet)

WifiRepeaterParametersSetResponse = _reflection.GeneratedProtocolMessageType('WifiRepeaterParametersSetResponse', (_message.Message,), dict(
  DESCRIPTOR = _WIFIREPEATERPARAMETERSSETRESPONSE,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterParametersSetResponse)
  ))
_sym_db.RegisterMessage(WifiRepeaterParametersSetResponse)

WifiWpsPbcStart = _reflection.GeneratedProtocolMessageType('WifiWpsPbcStart', (_message.Message,), dict(
  DESCRIPTOR = _WIFIWPSPBCSTART,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiWpsPbcStart)
  ))
_sym_db.RegisterMessage(WifiWpsPbcStart)

WifiRepeaterWpsClonePbcStart = _reflection.GeneratedProtocolMessageType('WifiRepeaterWpsClonePbcStart', (_message.Message,), dict(
  DESCRIPTOR = _WIFIREPEATERWPSCLONEPBCSTART,
  __module__ = 'devolo_idl_proto_deviceapi_wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterWpsClonePbcStart)
  ))
_sym_db.RegisterMessage(WifiRepeaterWpsClonePbcStart)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\006deviceB\013wifinetwork'))
# @@protoc_insertion_point(module_scope)



import asyncio
import json
import logging
import pathlib

from httpx import AsyncClient

from devolo_plc_api.clients.protobuf import Protobuf

file = pathlib.Path(__file__).parent / "../test_data.json"
with file.open("r") as fh:
    test_data = json.load(fh)


class StubProtobuf(Protobuf):

    def __init__(self):
        self._logger = logging.getLogger("ProtobufMock")
        self._loop = asyncio.new_event_loop()
        self._ip = test_data["ip"]
        self._port = 14791
        self._session = AsyncClient()
        self._path = test_data["device_info"]["_dvl-plcnetapi._tcp.local."]["properties"]["Path"]
        self._version = test_data["device_info"]["_dvl-plcnetapi._tcp.local."]["properties"]["Version"]
        self._user = "user"

        self.password = "password"



import asyncio
import ipaddress
import logging
import struct
from datetime import date
from typing import Dict, Optional

import httpx
from zeroconf import ServiceBrowser, ServiceInfo, ServiceStateChange, Zeroconf

from .device_api.deviceapi import DeviceApi
from .exceptions.device import DeviceNotFound
from .plcnet_api.plcnetapi import PlcNetApi

EMPTY_INFO: Dict = {
    "properties": {}
}


class Device:
    """
    Representing object for your devolo PLC device. It stores all properties and functionalities discovered during setup.

    :param ip: IP address of the device to communicate with.
    :param plcnetapi: Reuse externally gathered data for the plcnet API
    :param deviceapi: Reuse externally gathered data for the device API
    :param zeroconf_instance: Zeroconf instance to be potentially reused.
    """

    def __init__(self,
                 ip: str,
                 plcnetapi: Optional[Dict] = None,
                 deviceapi: Optional[Dict] = None,
                 zeroconf_instance: Optional[Zeroconf] = None):
        self.firmware_date = date.fromtimestamp(0)
        self.firmware_version = ""
        self.hostname = ""
        self.ip = ip
        self.mac = ""
        self.mt_number = 0
        self.product = ""
        self.technology = ""
        self.serial_number = 0

        self.device = None
        self.plcnet = None

        self._connected = False
        self._info: Dict = {
            "_dvl-plcnetapi._tcp.local.": plcnetapi or EMPTY_INFO,
            "_dvl-deviceapi._tcp.local.": deviceapi or EMPTY_INFO,
        }
        self._logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")
        self._password = ""
        self._session_instance: Optional[httpx.AsyncClient] = None
        self._zeroconf_instance = zeroconf_instance
        logging.captureWarnings(True)

        self._loop: asyncio.AbstractEventLoop
        self._session: httpx.AsyncClient
        self._zeroconf: Zeroconf

    def __del__(self):
        if self._connected and self._session_instance is None:
            self._logger.warning("Please disconnect properly from the device.")

    async def __aenter__(self):
        await self.async_connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.async_disconnect()

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()

    @property
    def password(self) -> str:
        """ The currently set device password. """
        return self._password

    @password.setter
    def password(self, password: str):
        """ Change the currently set device password. """
        self._password = password
        if self.device:
            self.device.password = password

    async def async_connect(self, session_instance: Optional[httpx.AsyncClient] = None):
        """
        Connect to a device asynchronous.

        :param: session_instance: Session client instance to be potentially reused.
        """
        self._loop = asyncio.get_running_loop()
        self._session_instance = session_instance
        self._session = self._session_instance or httpx.AsyncClient()
        self._zeroconf = self._zeroconf_instance or Zeroconf()
        await asyncio.gather(self._get_device_info(), self._get_plcnet_info())
        if not self.device and not self.plcnet:
            raise DeviceNotFound(f"The device {self.ip} did not answer.")
        self._connected = True

    def connect(self):
        """ Connect to a device synchronous. """
        self._loop = asyncio.new_event_loop()
        self._loop.run_until_complete(self.async_connect())

    async def async_disconnect(self):
        """ Disconnect from a device asynchronous. """
        if not self._zeroconf_instance:
            self._zeroconf.close()
        if not self._session_instance:
            await self._session.aclose()
        self._connected = False

    def disconnect(self):
        """ Disconnect from a device asynchronous. """
        self._loop.run_until_complete(self.async_disconnect())
        self._loop.close()

    async def _get_device_info(self):
        """ Get information from the devolo Device API. """
        service_type = "_dvl-deviceapi._tcp.local."
        try:
            await asyncio.wait_for(self._get_zeroconf_info(service_type=service_type), timeout=3)
        except asyncio.TimeoutError:
            return

        self.firmware_date = date.fromisoformat(self._info[service_type]["properties"].get("FirmwareDate", "1970-01-01"))
        self.firmware_version = self._info[service_type]["properties"].get("FirmwareVersion", "")
        self.hostname = self._info[service_type].get("hostname", "")
        self.mt_number = self._info[service_type]["properties"].get("MT", 0)
        self.product = self._info[service_type]["properties"].get("Product", "")
        self.serial_number = self._info[service_type]["properties"]["SN"]

        self.device = DeviceApi(ip=self.ip, session=self._session, info=self._info[service_type])

    async def _get_plcnet_info(self):
        """ Get information from the devolo PlcNet API. """
        service_type = "_dvl-plcnetapi._tcp.local."
        try:
            await asyncio.wait_for(self._get_zeroconf_info(service_type=service_type), timeout=3)
        except asyncio.TimeoutError:
            return

        self.mac = self._info[service_type]["properties"]["PlcMacAddress"]
        self.technology = self._info[service_type]["properties"].get("PlcTechnology", "")

        self.plcnet = PlcNetApi(ip=self.ip, session=self._session, info=self._info[service_type])

    async def _get_zeroconf_info(self, service_type: str):
        """ Browse for the desired mDNS service types and query them. """
        if self._info[service_type]["properties"]:
            return  # No need to continue, if device info already exist

        self._logger.debug("Browsing for %s", service_type)
        browser = ServiceBrowser(self._zeroconf, service_type, [self._state_change], addr=self.ip)
        while not self._info[service_type]["properties"]:
            await asyncio.sleep(0.1)
        browser.cancel()

    def _state_change(self, zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange):
        """ Evaluate the query result. """
        service_info = zeroconf.get_service_info(service_type, name)

        if service_info is None or str(ipaddress.ip_address(service_info.addresses[0])) != self.ip:
            return  # No need to continue, if there are no relevant service information

        if state_change is ServiceStateChange.Added:
            self._logger.debug("Adding service info of %s", service_type)
            self._info[service_type] = self.info_from_service(service_info)

    @staticmethod
    def info_from_service(service_info: ServiceInfo) -> Optional[Dict]:
        """ Return prepared info from mDNS entries. """
        properties = {}
        if not service_info.addresses:
            return None  # No need to continue, if there is no IP address to contact the device

        total_length = len(service_info.text)
        offset = 0
        while offset < total_length:
            parsed_length, = struct.unpack_from("!B", service_info.text, offset)
            key_value = service_info.text[offset + 1:offset + 1 + parsed_length].decode("UTF-8").split("=")
            properties[key_value[0]] = key_value[1]
            offset += parsed_length + 1

        address = service_info.addresses[0]

        return {
            "address": str(ipaddress.ip_address(address)),
            "hostname": service_info.server,
            "port": service_info.port,
            "properties": properties,
        }



import json
import pathlib

import pytest

pytest_plugins = [
    'tests.fixtures.device',
    'tests.fixtures.device_api',
    'tests.fixtures.plcnet_api',
    'tests.fixtures.protobuf',
]

file = pathlib.Path(__file__).parent / "test_data.json"
with file.open("r") as fh:
    test_data = json.load(fh)


@pytest.fixture(autouse=True, scope="class")
def test_data_fixture(request):
    """ Load test data. """
    request.cls.device_info = test_data['device_info']
    request.cls.ip = test_data['ip']



import pytest
from google.protobuf.json_format import MessageToDict

from devolo_plc_api.plcnet_api.devolo_idl_proto_plcnetapi_getnetworkoverview_pb2 import GetNetworkOverview
from devolo_plc_api.plcnet_api.devolo_idl_proto_plcnetapi_identifydevice_pb2 import IdentifyDeviceResponse
from devolo_plc_api.plcnet_api.devolo_idl_proto_plcnetapi_setuserdevicename_pb2 import SetUserDeviceNameResponse


class TestDeviceApi:

    @pytest.mark.asyncio
    async def test_async_get_network_overview(self, plcnet_api, httpx_mock):
        network_overview = GetNetworkOverview()
        httpx_mock.add_response(data=network_overview.SerializeToString())
        overview = await plcnet_api.async_get_network_overview()
        assert overview == MessageToDict(network_overview,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)

    def test_get_network_overview(self, plcnet_api, httpx_mock):
        network_overview = GetNetworkOverview()
        httpx_mock.add_response(data=network_overview.SerializeToString())
        overview = plcnet_api.get_network_overview()
        assert overview == MessageToDict(network_overview,
                                         including_default_value_fields=True,
                                         preserving_proto_field_name=True)

    @pytest.mark.asyncio
    async def test_async_identify_device_start(self, plcnet_api, httpx_mock):
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(data=identify_device.SerializeToString())
        assert await plcnet_api.async_identify_device_start()

    def test_identify_device_start(self, plcnet_api, httpx_mock):
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(data=identify_device.SerializeToString())
        assert plcnet_api.identify_device_start()

    @pytest.mark.asyncio
    async def test_async_identify_device_stop(self, plcnet_api, httpx_mock):
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(data=identify_device.SerializeToString())
        assert await plcnet_api.async_identify_device_stop()

    def test_identify_device_stop(self, plcnet_api, httpx_mock):
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(data=identify_device.SerializeToString())
        assert plcnet_api.identify_device_stop()

    @pytest.mark.asyncio
    async def test_async_set_user_device_name(self, plcnet_api, httpx_mock):
        user_device_name_set = SetUserDeviceNameResponse()
        httpx_mock.add_response(data=user_device_name_set.SerializeToString())
        assert await plcnet_api.async_set_user_device_name("Test")

    def test_set_user_device_name(self, plcnet_api, httpx_mock):
        user_device_name_set = SetUserDeviceNameResponse()
        httpx_mock.add_response(data=user_device_name_set.SerializeToString())
        assert plcnet_api.set_user_device_name("Test")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: devolo_idl_proto_deviceapi_ledsettings.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='devolo_idl_proto_deviceapi_ledsettings.proto',
  package='device.api',
  syntax='proto3',
  serialized_pb=_b('\n,devolo_idl_proto_deviceapi_ledsettings.proto\x12\ndevice.api\"i\n\x0eLedSettingsGet\x12\x32\n\x05state\x18\x01 \x01(\x0e\x32#.device.api.LedSettingsGet.Ledstate\"#\n\x08Ledstate\x12\n\n\x06LED_ON\x10\x00\x12\x0b\n\x07LED_OFF\x10\x01\"i\n\x0eLedSettingsSet\x12\x32\n\x05state\x18\x01 \x01(\x0e\x32#.device.api.LedSettingsSet.Ledstate\"#\n\x08Ledstate\x12\n\n\x06LED_ON\x10\x00\x12\x0b\n\x07LED_OFF\x10\x01\"~\n\x16LedSettingsSetResponse\x12\x39\n\x06result\x18\x01 \x01(\x0e\x32).device.api.LedSettingsSetResponse.Result\")\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x0e\n\x06\x64\x65viceB\x04ledsb\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_LEDSETTINGSGET_LEDSTATE = _descriptor.EnumDescriptor(
  name='Ledstate',
  full_name='device.api.LedSettingsGet.Ledstate',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LED_ON', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LED_OFF', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=130,
  serialized_end=165,
)
_sym_db.RegisterEnumDescriptor(_LEDSETTINGSGET_LEDSTATE)

_LEDSETTINGSSET_LEDSTATE = _descriptor.EnumDescriptor(
  name='Ledstate',
  full_name='device.api.LedSettingsSet.Ledstate',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LED_ON', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LED_OFF', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=130,
  serialized_end=165,
)
_sym_db.RegisterEnumDescriptor(_LEDSETTINGSSET_LEDSTATE)

_LEDSETTINGSSETRESPONSE_RESULT = _descriptor.EnumDescriptor(
  name='Result',
  full_name='device.api.LedSettingsSetResponse.Result',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=1, number=255,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=359,
  serialized_end=400,
)
_sym_db.RegisterEnumDescriptor(_LEDSETTINGSSETRESPONSE_RESULT)


_LEDSETTINGSGET = _descriptor.Descriptor(
  name='LedSettingsGet',
  full_name='device.api.LedSettingsGet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='device.api.LedSettingsGet.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LEDSETTINGSGET_LEDSTATE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=165,
)


_LEDSETTINGSSET = _descriptor.Descriptor(
  name='LedSettingsSet',
  full_name='device.api.LedSettingsSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='device.api.LedSettingsSet.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LEDSETTINGSSET_LEDSTATE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=272,
)


_LEDSETTINGSSETRESPONSE = _descriptor.Descriptor(
  name='LedSettingsSetResponse',
  full_name='device.api.LedSettingsSetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.LedSettingsSetResponse.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LEDSETTINGSSETRESPONSE_RESULT,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=274,
  serialized_end=400,
)

_LEDSETTINGSGET.fields_by_name['state'].enum_type = _LEDSETTINGSGET_LEDSTATE
_LEDSETTINGSGET_LEDSTATE.containing_type = _LEDSETTINGSGET
_LEDSETTINGSSET.fields_by_name['state'].enum_type = _LEDSETTINGSSET_LEDSTATE
_LEDSETTINGSSET_LEDSTATE.containing_type = _LEDSETTINGSSET
_LEDSETTINGSSETRESPONSE.fields_by_name['result'].enum_type = _LEDSETTINGSSETRESPONSE_RESULT
_LEDSETTINGSSETRESPONSE_RESULT.containing_type = _LEDSETTINGSSETRESPONSE
DESCRIPTOR.message_types_by_name['LedSettingsGet'] = _LEDSETTINGSGET
DESCRIPTOR.message_types_by_name['LedSettingsSet'] = _LEDSETTINGSSET
DESCRIPTOR.message_types_by_name['LedSettingsSetResponse'] = _LEDSETTINGSSETRESPONSE

LedSettingsGet = _reflection.GeneratedProtocolMessageType('LedSettingsGet', (_message.Message,), dict(
  DESCRIPTOR = _LEDSETTINGSGET,
  __module__ = 'devolo_idl_proto_deviceapi_ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsGet)
  ))
_sym_db.RegisterMessage(LedSettingsGet)

LedSettingsSet = _reflection.GeneratedProtocolMessageType('LedSettingsSet', (_message.Message,), dict(
  DESCRIPTOR = _LEDSETTINGSSET,
  __module__ = 'devolo_idl_proto_deviceapi_ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsSet)
  ))
_sym_db.RegisterMessage(LedSettingsSet)

LedSettingsSetResponse = _reflection.GeneratedProtocolMessageType('LedSettingsSetResponse', (_message.Message,), dict(
  DESCRIPTOR = _LEDSETTINGSSETRESPONSE,
  __module__ = 'devolo_idl_proto_deviceapi_ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsSetResponse)
  ))
_sym_db.RegisterMessage(LedSettingsSetResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\006deviceB\004leds'))
# @@protoc_insertion_point(module_scope)



import pytest
from httpx import ConnectTimeout

from ..stubs.protobuf import StubProtobuf


@pytest.fixture()
async def mock_protobuf():
    protobuf = StubProtobuf()
    yield protobuf
    await protobuf._session.aclose()


@pytest.fixture()
def mock_device_unavailable(httpx_mock):

    def raise_type_error(request, extensions):
        raise ConnectTimeout(request=request, message=extensions)

    httpx_mock.add_callback(raise_type_error)


@pytest.fixture()
def mock_wrong_password(httpx_mock):

    def raise_type_error(request, extensions):
        raise TypeError

    httpx_mock.add_callback(raise_type_error)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: updatefirmware.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='updatefirmware.proto',
  package='device.api',
  syntax='proto3',
  serialized_options=_b('\n\006deviceB\006update'),
  serialized_pb=_b('\n\x14updatefirmware.proto\x12\ndevice.api\"\xb9\x01\n\x13UpdateFirmwareCheck\x12\x36\n\x06result\x18\x01 \x01(\x0e\x32&.device.api.UpdateFirmwareCheck.Result\x12\x1c\n\x14new_firmware_version\x18\x02 \x01(\t\"L\n\x06Result\x12\x14\n\x10UPDATE_AVAILABLE\x10\x00\x12\x18\n\x14UPDATE_NOT_AVAILABLE\x10\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\"\x99\x01\n\x13UpdateFirmwareStart\x12\x36\n\x06result\x18\x01 \x01(\x0e\x32&.device.api.UpdateFirmwareStart.Result\"J\n\x06Result\x12\x12\n\x0eUPDATE_STARTED\x10\x00\x12\x18\n\x14UPDATE_NOT_AVAILABLE\x10\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x10\n\x06\x64\x65viceB\x06updateb\x06proto3')
)



_UPDATEFIRMWARECHECK_RESULT = _descriptor.EnumDescriptor(
  name='Result',
  full_name='device.api.UpdateFirmwareCheck.Result',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UPDATE_AVAILABLE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATE_NOT_AVAILABLE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=2, number=255,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=146,
  serialized_end=222,
)
_sym_db.RegisterEnumDescriptor(_UPDATEFIRMWARECHECK_RESULT)

_UPDATEFIRMWARESTART_RESULT = _descriptor.EnumDescriptor(
  name='Result',
  full_name='device.api.UpdateFirmwareStart.Result',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UPDATE_STARTED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATE_NOT_AVAILABLE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=2, number=255,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=304,
  serialized_end=378,
)
_sym_db.RegisterEnumDescriptor(_UPDATEFIRMWARESTART_RESULT)


_UPDATEFIRMWARECHECK = _descriptor.Descriptor(
  name='UpdateFirmwareCheck',
  full_name='device.api.UpdateFirmwareCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.UpdateFirmwareCheck.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='new_firmware_version', full_name='device.api.UpdateFirmwareCheck.new_firmware_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPDATEFIRMWARECHECK_RESULT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=37,
  serialized_end=222,
)


_UPDATEFIRMWARESTART = _descriptor.Descriptor(
  name='UpdateFirmwareStart',
  full_name='device.api.UpdateFirmwareStart',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='device.api.UpdateFirmwareStart.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPDATEFIRMWARESTART_RESULT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=225,
  serialized_end=378,
)

_UPDATEFIRMWARECHECK.fields_by_name['result'].enum_type = _UPDATEFIRMWARECHECK_RESULT
_UPDATEFIRMWARECHECK_RESULT.containing_type = _UPDATEFIRMWARECHECK
_UPDATEFIRMWARESTART.fields_by_name['result'].enum_type = _UPDATEFIRMWARESTART_RESULT
_UPDATEFIRMWARESTART_RESULT.containing_type = _UPDATEFIRMWARESTART
DESCRIPTOR.message_types_by_name['UpdateFirmwareCheck'] = _UPDATEFIRMWARECHECK
DESCRIPTOR.message_types_by_name['UpdateFirmwareStart'] = _UPDATEFIRMWARESTART
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UpdateFirmwareCheck = _reflection.GeneratedProtocolMessageType('UpdateFirmwareCheck', (_message.Message,), dict(
  DESCRIPTOR = _UPDATEFIRMWARECHECK,
  __module__ = 'updatefirmware_pb2'
  # @@protoc_insertion_point(class_scope:device.api.UpdateFirmwareCheck)
  ))
_sym_db.RegisterMessage(UpdateFirmwareCheck)

UpdateFirmwareStart = _reflection.GeneratedProtocolMessageType('UpdateFirmwareStart', (_message.Message,), dict(
  DESCRIPTOR = _UPDATEFIRMWARESTART,
  __module__ = 'updatefirmware_pb2'
  # @@protoc_insertion_point(class_scope:device.api.UpdateFirmwareStart)
  ))
_sym_db.RegisterMessage(UpdateFirmwareStart)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import asyncio
import logging
from abc import ABC, abstractclassmethod
from typing import Callable

from google.protobuf.json_format import MessageToDict
from httpx import AsyncClient, ConnectError, ConnectTimeout, DigestAuth, ReadTimeout, Response

from ..exceptions.device import DevicePasswordProtected, DeviceUnavailable

TIMEOUT = 5.0


class Protobuf(ABC):
    """
    Google Protobuf client.
    """

    @abstractclassmethod
    def __init__(self):
        self._loop = asyncio.get_running_loop()
        self._logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")

        self.password: str

        self._ip: str
        self._path: str
        self._port: int
        self._session: AsyncClient
        self._user: str
        self._version: str

    def __getattr__(self, attr: str) -> Callable:
        """ Catch attempts to call methods synchronously. """

        def method(*args, **kwargs):
            return self._loop.run_until_complete(getattr(self, async_method)(*args, **kwargs))

        async_method = f"async_{attr}"
        if hasattr(self.__class__, async_method):
            return method
        raise AttributeError(f"{self.__class__.__name__} object has no attribute {attr}")

    @property
    def url(self) -> str:
        """ The base URL to query. """
        return f"http://{self._ip}:{self._port}/{self._path}/{self._version}/"

    async def _async_get(self, sub_url: str, timeout: float = TIMEOUT) -> Response:
        """ Query URL asynchronously. """
        url = f"{self.url}{sub_url}"
        self._logger.debug("Getting from %s", url)
        try:
            return await self._session.get(url, auth=DigestAuth(self._user, self.password), timeout=timeout)
        except TypeError:
            raise DevicePasswordProtected("The used password is wrong.") from None
        except (ConnectTimeout, ConnectError, ReadTimeout):
            raise DeviceUnavailable("The device is currenctly not available. Maybe on standby?") from None

    async def _async_post(self, sub_url: str, content: bytes, timeout: float = TIMEOUT) -> Response:
        """ Post data asynchronously. """
        url = f"{self.url}{sub_url}"
        self._logger.debug("Posting to %s", url)
        try:
            return await self._session.post(url, auth=DigestAuth(self._user, self.password), content=content, timeout=timeout)
        except TypeError:
            raise DevicePasswordProtected("The used password is wrong.") from None
        except (ConnectTimeout, ConnectError, ReadTimeout):
            raise DeviceUnavailable("The device is currenctly not available. Maybe on standby?") from None

    @staticmethod
    def _message_to_dict(message) -> dict:
        """ Convert message to dict with certain settings. """
        return MessageToDict(message=message, including_default_value_fields=True, preserving_proto_field_name=True)



from typing import Dict

from httpx import AsyncClient

from ..clients.protobuf import Protobuf
from . import (devolo_idl_proto_plcnetapi_getnetworkoverview_pb2,
               devolo_idl_proto_plcnetapi_identifydevice_pb2,
               devolo_idl_proto_plcnetapi_setuserdevicename_pb2)


class PlcNetApi(Protobuf):
    """
    Implementation of the devolo plcnet API.

    :param ip: IP address of the device to communicate with
    :param session: HTTP client session
    :param info: Information collected from the mDNS query
    """

    def __init__(self, ip: str, session: AsyncClient, info: Dict):
        super().__init__()

        self._ip = ip
        self._mac = info["properties"]["PlcMacAddress"]
        self._path = info["properties"]["Path"]
        self._port = info["port"]
        self._session = session
        self._user = ""  # PLC API is not password protected.
        self._version = info["properties"]["Version"]

        self.password = ""  # PLC API is not password protected.

    async def async_get_network_overview(self) -> dict:
        """
        Get a PLC network overview.

        :return: Network overview
        """
        self._logger.debug("Getting network overview.")
        network_overview = devolo_idl_proto_plcnetapi_getnetworkoverview_pb2.GetNetworkOverview()
        response = await self._async_get("GetNetworkOverview")
        network_overview.ParseFromString(await response.aread())
        return self._message_to_dict(network_overview)

    async def async_identify_device_start(self) -> bool:
        """
        Make PLC LED of a device blick to identify it.

        :return: True, if identifying was successfully started, otherwise False
        """
        self._logger.debug("Starting LED blinking.")
        identify_device = devolo_idl_proto_plcnetapi_identifydevice_pb2.IdentifyDeviceStart()
        identify_device.mac_address = self._mac
        query = await self._async_post("IdentifyDeviceStart", content=identify_device.SerializeToString())
        response = devolo_idl_proto_plcnetapi_identifydevice_pb2.IdentifyDeviceResponse()
        response.FromString(await query.aread())  # pylint: disable=no-member
        return bool(not response.result)  # pylint: disable=no-member

    async def async_identify_device_stop(self) -> bool:
        """
        Stop the PLC LED blicking.

        :return: True, if identifying was successfully stopped, otherwise False
        """
        self._logger.debug("Stopping LED blinking.")
        identify_device = devolo_idl_proto_plcnetapi_identifydevice_pb2.IdentifyDeviceStop()
        identify_device.mac_address = self._mac
        query = await self._async_post("IdentifyDeviceStop", content=identify_device.SerializeToString())
        response = devolo_idl_proto_plcnetapi_identifydevice_pb2.IdentifyDeviceResponse()
        response.FromString(await query.aread())  # pylint: disable=no-member
        return bool(not response.result)  # pylint: disable=no-member

    async def async_set_user_device_name(self, name) -> bool:
        """
        Set device name.

        :param name: Name, the device shall have
        :return: True, if the device was successfully renamed, otherwise False
        """
        self._logger.debug("Setting device name.")
        set_user_name = devolo_idl_proto_plcnetapi_setuserdevicename_pb2.SetUserDeviceName()
        set_user_name.mac_address = self._mac
        set_user_name.user_device_name = name
        query = await self._async_post("SetUserDeviceName", content=set_user_name.SerializeToString(), timeout=10.0)
        response = devolo_idl_proto_plcnetapi_setuserdevicename_pb2.SetUserDeviceNameResponse()
        response.FromString(await query.aread())  # pylint: disable=no-member
        return bool(not response.result)  # pylint: disable=no-member



import asyncio
from unittest.mock import patch

import pytest
from httpx import AsyncClient

from devolo_plc_api.device_api.deviceapi import DeviceApi


@pytest.fixture()
async def device_api(request, feature):
    with patch("asyncio.get_running_loop", asyncio.new_event_loop):
        request.cls.device_info["_dvl-deviceapi._tcp.local."]["properties"]["Features"] = feature
        async with AsyncClient() as client:
            device_api = DeviceApi(request.cls.ip, client, request.cls.device_info["_dvl-deviceapi._tcp.local."])
            yield device_api


@pytest.fixture()
def mock_device_api(mocker):
    mocker.patch("devolo_plc_api.device_api.deviceapi.DeviceApi")



class DeviceNotFound(Exception):
    """ The device was not found. """


class DeviceUnavailable(Exception):
    """ The device is not available, e.g. in standby. """


class DevicePasswordProtected(Exception):
    """ The device is passwort protected. """



import asyncio

from devolo_plc_api.device import Device

# IP of the device to query
IP = "192.168.0.10"

# Password, if the device has one. It is the same as the Web-UI has. It no password is set, you can remove the password
# parameter or set it to None.
PASSWORD = "super_secret"


async def run():
    async with Device(ip=IP) as dpa:
        # Set the password
        dpa.password = PASSWORD

        # Get LED settings of the device. The state might be LED_ON or LED_OFF.
        # {'state': 'LED_ON'}
        print(await dpa.device.async_get_led_setting())

        # Set LED settings of the device. Set enable to True to them turn on, to False to turn them off.
        # If the state was changed successfully, True is returned, otherwise False.
        print("success" if await dpa.device.async_set_led_setting(enable=True) else "failed")

        # Check for new firmware versions
        # {'result': 'UPDATE_NOT_AVAILABLE', 'new_firmware_version': ''}
        print(await dpa.device.async_check_firmware_available())

        # Start firmware update, if new version is available. Important: The response does not tell you anything about the
        # success of the update itself.
        print("update started" if await dpa.device.async_start_firmware_update() else "no update available")

        # Get details of wifi stations connected to the device: MAC address, access point type (main or guest), wifi band and
        # connection rates.
        # {'connected_stations':
        #   [
        #     {'mac_address': 'AA:BB:CC:DD:EE:FF',
        #      'vap_type': 'WIFI_VAP_MAIN_AP',
        #      'band': 'WIFI_BAND_5G',
        #      'rx_rate': 87800,
        #      'tx_rate': 87800}
        #   ]
        # }
        print(await dpa.device.async_get_wifi_connected_station())

        # Get details about wifi guest access: SSID, Wifi key, state (enabled/disabled) and if time limited, the remaining
        # duration.
        # {'ssid': 'devolo-guest-930', 'key': 'HMANPGBA', 'enabled': False, 'remaining_duration': 0}
        print(await dpa.device.async_get_wifi_guest_access())

        # Enable or disable the wifi guest access. Set enable to True to it turn on, to False to turn it off. Chaning SSID,
        # wifi key or duration is currently not supported. If the state was changed successfully, True is returned, otherwise
        # False.
        print("success" if await dpa.device.async_set_wifi_guest_access(enable=False) else "failed")

        # Get details about other access points in your neighborhood: MAC address, SSID, wifi band, used channel, signal
        # strength in DB and a value from 1 to 5, if you would want to map the signal strenght to a signal bars.
        # {'neighbor_aps':
        #   [
        #     {'mac_address': 'AA:BB:CC:DD:EE:FF',
        #      'ssid': 'wifi',
        #      'band': 'WIFI_BAND_2G',
        #      'channel': 1,
        #      'signal': -73,
        #      'signal_bars': 1}
        #   ]
        # }
        print(await dpa.device.async_get_wifi_neighbor_access_points())

        # Start WPS push button configuration. If WPS was started successfully, True is returned, otherwise False.
        print("WPS started" if await dpa.device.async_start_wps() else "WPS start failed")

        # Get PLC network overview with enriched information like firmware version,
        # {'network':
        #   {'devices':
        #       [
        #         {'product_name': 'devolo dLAN pro 1200+ WiFi ac',
        #          'product_id': 'MT2730',
        #          'friendly_version': '2.8.0.01',
        #          'full_version': 'MAC-QCA7500-2.8.0.30-01-20190705-CS',
        #          'user_device_name': '',
        #          'mac_address': 'AABBCCDDEEFF',
        #          'topology': 'LOCAL',
        #          'technology': 'HPAV_PANTHER',
        #          'bridged_devices': [],
        #          'user_network_name': '',
        #          'ipv4_address': '',
        #          'attached_to_router': False},
        #         {'product_name': 'devolo dLAN 1200+',
        #          'product_id': 'MT2639',
        #          'friendly_version': '2.8.0.01-1',
        #          'full_version': 'MAC-QCA7500-2.8.0.30-01-20190705-CS',
        #          'user_device_name': '',
        #          'mac_address': 'AABBCCDDEEFF',
        #          'topology': 'REMOTE',
        #          'technology': 'HPAV_PANTHER',
        #          'bridged_devices': [],
        #          'attached_to_router': True,
        #          'user_network_name': '',
        #          'ipv4_address': ''}
        #       ],
        #    'data_rates':
        #      [
        #        {'mac_address_from': 'AABBCCDDEEFF',
        #         'mac_address_to': 'AABBCCDDEEFF',
        #         'tx_rate': 129.9375,
        #         'rx_rate': 124.6875}
        #      ]
        #   }
        # }
        print(await dpa.plcnet.async_get_network_overview())

        # Identify the device by making the PLC LED blink. This call returns directly with True, if identifing was started
        # succcessfully, otherwise False. However, the LED stays blinking for two minutes.
        print("success" if await dpa.plcnet.async_identify_device_start() else "failed")

        # Stop identify the device if you don't want to wait for the timeout.
        print("success" if await dpa.plcnet.async_identify_device_start() else "failed")

        # Set the user device name. If the name was changed successfully, True is returned, otherwise False.
        print("success" if await dpa.plcnet.async_set_user_device_name(name="New name") else "failed")


if __name__ == "__main__":
    asyncio.run(run())



import json
import pathlib

file = pathlib.Path(__file__).parent / ".." / "test_data.json"
with file.open("r") as fh:
    test_data = json.load(fh)


def state_change(self):
    self._info = test_data["device_info"]



from typing import Callable, Dict

from httpx import AsyncClient

from ..clients.protobuf import Protobuf
from ..exceptions.feature import FeatureNotSupported
from . import (devolo_idl_proto_deviceapi_ledsettings_pb2,
               devolo_idl_proto_deviceapi_updatefirmware_pb2,
               devolo_idl_proto_deviceapi_wifinetwork_pb2)


class DeviceApi(Protobuf):
    """
    Implementation of the devolo device API.

    :param ip: IP address of the device to communicate with
    :param session: HTTP client session
    :param info: Information collected from the mDNS query
    """

    def __init__(self, ip: str, session: AsyncClient, info: Dict):
        super().__init__()

        self._ip = ip
        # HC gateway has no Path, it has a path.
        self._path = info["properties"].get("Path") or info["properties"].get("path")
        self._port = info["port"]
        self._session = session
        self._user = "devolo"
        self._version = info["properties"]["Version"]

        features = info["properties"].get("Features", "")
        self.features = features.split(",") if features else ["reset", "update", "led", "intmtg"]
        self.password = ""

    def _feature(feature: str):  # type: ignore  # pylint: disable=no-self-argument
        """ Decorator to filter unsupported features before querying the device. """

        def feature_decorator(method: Callable):

            def wrapper(self, *args, **kwargs):
                if feature in self.features:
                    return method(self, *args, **kwargs)
                else:
                    raise FeatureNotSupported(f"The device does not support {method}.")

            return wrapper

        return feature_decorator

    @_feature("led")
    async def async_get_led_setting(self) -> dict:
        """
        Get LED setting asynchronously. This feature only works on devices, that announce the led feature.

        return: LED settings
        """
        self._logger.debug("Getting LED settings.")
        led_setting = devolo_idl_proto_deviceapi_ledsettings_pb2.LedSettingsGet()
        response = await self._async_get("LedSettingsGet")
        led_setting.FromString(await response.aread())  # pylint: disable=no-member
        return self._message_to_dict(led_setting)

    @_feature("led")
    async def async_set_led_setting(self, enable: bool) -> bool:
        """
        Set LED setting asynchronously. This feature only works on devices, that announce the led feature.

        :param enable: True to enable the LEDs, False to disable the LEDs
        :return: True, if LED state was successfully changed, otherwise False
        """
        self._logger.debug("Setting LED settings.")
        led_setting = devolo_idl_proto_deviceapi_ledsettings_pb2.LedSettingsSet()
        led_setting.state = int(not enable)
        query = await self._async_post("LedSettingsSet", content=led_setting.SerializeToString())
        response = devolo_idl_proto_deviceapi_ledsettings_pb2.LedSettingsSetResponse()
        response.FromString(await query.aread())  # pylint: disable=no-member
        return bool(not response.result)  # pylint: disable=no-member

    @_feature("update")
    async def async_check_firmware_available(self) -> dict:
        """
        Check asynchronously, if a firmware update is available for the device.

        :return: Result and new firmware version, if newer one is available
        """
        self._logger.debug("Checking for new firmware.")
        update_firmware_check = devolo_idl_proto_deviceapi_updatefirmware_pb2.UpdateFirmwareCheck()
        response = await self._async_get("UpdateFirmwareCheck")
        update_firmware_check.ParseFromString(await response.aread())
        return self._message_to_dict(update_firmware_check)

    @_feature("update")
    async def async_start_firmware_update(self) -> bool:
        """
        Start firmware update asynchronously, if a firmware update is available for the device. Important: The response does
        not tell you anything about the success of the update itself.

        :return: True, if the firmware update was started, False if there is no update
        """
        self._logger.debug("Updating firmware.")
        update_firmware = devolo_idl_proto_deviceapi_updatefirmware_pb2.UpdateFirmwareStart()
        response = await self._async_get("UpdateFirmwareStart")
        update_firmware.FromString(await response.aread())  # pylint: disable=no-member
        return bool(not update_firmware.result)  # pylint: disable=no-member

    @_feature("wifi1")
    async def async_get_wifi_connected_station(self) -> dict:
        """
        Get wifi stations connected to the device asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: All connected wifi stations including connection rate data
        """
        self._logger.debug("Getting connected wifi stations.")
        wifi_connected_proto = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiConnectedStationsGet()
        response = await self._async_get("WifiConnectedStationsGet")
        wifi_connected_proto.ParseFromString(await response.aread())
        return self._message_to_dict(wifi_connected_proto)

    @_feature("wifi1")
    async def async_get_wifi_guest_access(self) -> dict:
        """
        Get details about wifi guest access asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: Details about the wifi guest access
        """
        self._logger.debug("Getting wifi guest access status.")
        wifi_guest_proto = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiGuestAccessGet()
        response = await self._async_get("WifiGuestAccessGet")
        wifi_guest_proto.ParseFromString(await response.aread())
        return self._message_to_dict(wifi_guest_proto)

    @_feature("wifi1")
    async def async_set_wifi_guest_access(self, enable: bool) -> bool:
        """
        Enable wifi guest access asynchronously. This feature only works on devices, that announce the wifi1 feature.

        :param enable: True to enable, False to disable wifi guest access
        :return: True, if the state of the wifi guest access was successfully changed, otherwise False
        """
        self._logger.debug("Setting wifi guest access status.")
        wifi_guest_proto = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiGuestAccessSet()
        wifi_guest_proto.enable = enable
        query = await self._async_post("WifiGuestAccessSet", content=wifi_guest_proto.SerializeToString())
        response = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiGuestAccessSetResponse()
        response.FromString(await query.aread())  # pylint: disable=no-member
        return bool(not response.result)  # pylint: disable=no-member

    @_feature("wifi1")
    async def async_get_wifi_neighbor_access_points(self) -> dict:
        """
        Get wifi access point in the neighborhood asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: Visible access points in the neighborhood including connection rate data
        """
        self._logger.debug("Getting neighbored access points.")
        wifi_neighbor_aps = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiNeighborAPsGet()
        response = await self._async_get("WifiNeighborAPsGet", timeout=15.0)
        wifi_neighbor_aps.ParseFromString(await response.aread())
        return self._message_to_dict(wifi_neighbor_aps)

    @_feature("wifi1")
    async def async_get_wifi_repeated_access_points(self):
        """
        Get repeated wifi access point asynchronously. This feature only works on repeater devices, that announce the wifi1
        feature.

        :return: Repeated access points in the neighborhood including connection rate data
        """
        self._logger.debug("Getting repeated access points.")
        wifi_connected_proto = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiRepeatedAPsGet()
        response = await self._async_get("WifiRepeatedAPsGet")
        wifi_connected_proto.ParseFromString(await response.aread())
        return self._message_to_dict(wifi_connected_proto)

    @_feature("wifi1")
    async def async_start_wps(self):
        """
        Start WPS push button configuration.

        :return: True, if the WPS was successfully started, otherwise False
        """
        self._logger.debug("Starting WPS.")
        wps_proto = devolo_idl_proto_deviceapi_wifinetwork_pb2.WifiWpsPbcStart()
        response = await self._async_get("WifiWpsPbcStart")
        wps_proto.FromString(await response.aread())  # pylint: disable=no-member
        return bool(not wps_proto.result)  # pylint: disable=no-member



import asyncio
from unittest.mock import patch

import pytest
from httpx import AsyncClient

from devolo_plc_api.plcnet_api.plcnetapi import PlcNetApi


@pytest.fixture()
async def plcnet_api(request):
    with patch("asyncio.get_running_loop", asyncio.new_event_loop):
        asyncio.new_event_loop()
        async with AsyncClient() as client:
            plcnet_api = PlcNetApi(request.cls.ip, client, request.cls.device_info["_dvl-plcnetapi._tcp.local."])
            yield plcnet_api


@pytest.fixture()
def mock_plcnet_api(mocker):
    mocker.patch("devolo_plc_api.plcnet_api.plcnetapi.PlcNetApi")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: devolo_idl_proto_plcnetapi_getnetworkoverview.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='devolo_idl_proto_plcnetapi_getnetworkoverview.proto',
  package='plcnet.api',
  syntax='proto3',
  serialized_pb=_b('\n3devolo_idl_proto_plcnetapi_getnetworkoverview.proto\x12\nplcnet.api\"\xd5\x06\n\x12GetNetworkOverview\x12>\n\x07network\x18\x01 \x01(\x0b\x32-.plcnet.api.GetNetworkOverview.LogicalNetwork\x1a\x96\x04\n\x06\x44\x65vice\x12\x14\n\x0cproduct_name\x18\x01 \x01(\t\x12\x12\n\nproduct_id\x18\x02 \x01(\t\x12\x18\n\x10\x66riendly_version\x18\x03 \x01(\t\x12\x14\n\x0c\x66ull_version\x18\x04 \x01(\t\x12\x18\n\x10user_device_name\x18\x05 \x01(\t\x12\x19\n\x11user_network_name\x18\x06 \x01(\t\x12\x13\n\x0bmac_address\x18\x07 \x01(\t\x12@\n\x08topology\x18\x08 \x01(\x0e\x32..plcnet.api.GetNetworkOverview.Device.Topology\x12\x44\n\ntechnology\x18\t \x01(\x0e\x32\x30.plcnet.api.GetNetworkOverview.Device.Technology\x12\x17\n\x0f\x62ridged_devices\x18\n \x03(\t\x12\x14\n\x0cipv4_address\x18\x0b \x01(\t\x12\x1a\n\x12\x61ttached_to_router\x18\x0c \x01(\x08\"7\n\x08Topology\x12\x14\n\x10UNKNOWN_TOPOLOGY\x10\x00\x12\t\n\x05LOCAL\x10\x01\x12\n\n\x06REMOTE\x10\x02\"\\\n\nTechnology\x12\x16\n\x12UNKNOWN_TECHNOLOGY\x10\x00\x12\x14\n\x10HPAV_THUNDERBOLT\x10\x03\x12\x10\n\x0cHPAV_PANTHER\x10\x04\x12\x0e\n\nGHN_SPIRIT\x10\x07\x1a^\n\x08\x44\x61taRate\x12\x18\n\x10mac_address_from\x18\x01 \x01(\t\x12\x16\n\x0emac_address_to\x18\x02 \x01(\t\x12\x0f\n\x07tx_rate\x18\x03 \x01(\x01\x12\x0f\n\x07rx_rate\x18\x04 \x01(\x01\x1a\x85\x01\n\x0eLogicalNetwork\x12\x36\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32%.plcnet.api.GetNetworkOverview.Device\x12;\n\ndata_rates\x18\x02 \x03(\x0b\x32\'.plcnet.api.GetNetworkOverview.DataRateB\r\n\x06plcnetB\x03netb\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_GETNETWORKOVERVIEW_DEVICE_TOPOLOGY = _descriptor.EnumDescriptor(
  name='Topology',
  full_name='plcnet.api.GetNetworkOverview.Device.Topology',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_TOPOLOGY', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REMOTE', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=540,
  serialized_end=595,
)
_sym_db.RegisterEnumDescriptor(_GETNETWORKOVERVIEW_DEVICE_TOPOLOGY)

_GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY = _descriptor.EnumDescriptor(
  name='Technology',
  full_name='plcnet.api.GetNetworkOverview.Device.Technology',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_TECHNOLOGY', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HPAV_THUNDERBOLT', index=1, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HPAV_PANTHER', index=2, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GHN_SPIRIT', index=3, number=7,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=597,
  serialized_end=689,
)
_sym_db.RegisterEnumDescriptor(_GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY)


_GETNETWORKOVERVIEW_DEVICE = _descriptor.Descriptor(
  name='Device',
  full_name='plcnet.api.GetNetworkOverview.Device',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='product_name', full_name='plcnet.api.GetNetworkOverview.Device.product_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='product_id', full_name='plcnet.api.GetNetworkOverview.Device.product_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='friendly_version', full_name='plcnet.api.GetNetworkOverview.Device.friendly_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='full_version', full_name='plcnet.api.GetNetworkOverview.Device.full_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_device_name', full_name='plcnet.api.GetNetworkOverview.Device.user_device_name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_network_name', full_name='plcnet.api.GetNetworkOverview.Device.user_network_name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='plcnet.api.GetNetworkOverview.Device.mac_address', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='topology', full_name='plcnet.api.GetNetworkOverview.Device.topology', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='technology', full_name='plcnet.api.GetNetworkOverview.Device.technology', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bridged_devices', full_name='plcnet.api.GetNetworkOverview.Device.bridged_devices', index=9,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ipv4_address', full_name='plcnet.api.GetNetworkOverview.Device.ipv4_address', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='attached_to_router', full_name='plcnet.api.GetNetworkOverview.Device.attached_to_router', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETNETWORKOVERVIEW_DEVICE_TOPOLOGY,
    _GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=689,
)

_GETNETWORKOVERVIEW_DATARATE = _descriptor.Descriptor(
  name='DataRate',
  full_name='plcnet.api.GetNetworkOverview.DataRate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address_from', full_name='plcnet.api.GetNetworkOverview.DataRate.mac_address_from', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mac_address_to', full_name='plcnet.api.GetNetworkOverview.DataRate.mac_address_to', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tx_rate', full_name='plcnet.api.GetNetworkOverview.DataRate.tx_rate', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rx_rate', full_name='plcnet.api.GetNetworkOverview.DataRate.rx_rate', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=691,
  serialized_end=785,
)

_GETNETWORKOVERVIEW_LOGICALNETWORK = _descriptor.Descriptor(
  name='LogicalNetwork',
  full_name='plcnet.api.GetNetworkOverview.LogicalNetwork',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='devices', full_name='plcnet.api.GetNetworkOverview.LogicalNetwork.devices', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data_rates', full_name='plcnet.api.GetNetworkOverview.LogicalNetwork.data_rates', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=921,
)

_GETNETWORKOVERVIEW = _descriptor.Descriptor(
  name='GetNetworkOverview',
  full_name='plcnet.api.GetNetworkOverview',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='network', full_name='plcnet.api.GetNetworkOverview.network', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_GETNETWORKOVERVIEW_DEVICE, _GETNETWORKOVERVIEW_DATARATE, _GETNETWORKOVERVIEW_LOGICALNETWORK, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=921,
)

_GETNETWORKOVERVIEW_DEVICE.fields_by_name['topology'].enum_type = _GETNETWORKOVERVIEW_DEVICE_TOPOLOGY
_GETNETWORKOVERVIEW_DEVICE.fields_by_name['technology'].enum_type = _GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY
_GETNETWORKOVERVIEW_DEVICE.containing_type = _GETNETWORKOVERVIEW
_GETNETWORKOVERVIEW_DEVICE_TOPOLOGY.containing_type = _GETNETWORKOVERVIEW_DEVICE
_GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY.containing_type = _GETNETWORKOVERVIEW_DEVICE
_GETNETWORKOVERVIEW_DATARATE.containing_type = _GETNETWORKOVERVIEW
_GETNETWORKOVERVIEW_LOGICALNETWORK.fields_by_name['devices'].message_type = _GETNETWORKOVERVIEW_DEVICE
_GETNETWORKOVERVIEW_LOGICALNETWORK.fields_by_name['data_rates'].message_type = _GETNETWORKOVERVIEW_DATARATE
_GETNETWORKOVERVIEW_LOGICALNETWORK.containing_type = _GETNETWORKOVERVIEW
_GETNETWORKOVERVIEW.fields_by_name['network'].message_type = _GETNETWORKOVERVIEW_LOGICALNETWORK
DESCRIPTOR.message_types_by_name['GetNetworkOverview'] = _GETNETWORKOVERVIEW

GetNetworkOverview = _reflection.GeneratedProtocolMessageType('GetNetworkOverview', (_message.Message,), dict(

  Device = _reflection.GeneratedProtocolMessageType('Device', (_message.Message,), dict(
    DESCRIPTOR = _GETNETWORKOVERVIEW_DEVICE,
    __module__ = 'devolo_idl_proto_plcnetapi_getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.Device)
    ))
  ,

  DataRate = _reflection.GeneratedProtocolMessageType('DataRate', (_message.Message,), dict(
    DESCRIPTOR = _GETNETWORKOVERVIEW_DATARATE,
    __module__ = 'devolo_idl_proto_plcnetapi_getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.DataRate)
    ))
  ,

  LogicalNetwork = _reflection.GeneratedProtocolMessageType('LogicalNetwork', (_message.Message,), dict(
    DESCRIPTOR = _GETNETWORKOVERVIEW_LOGICALNETWORK,
    __module__ = 'devolo_idl_proto_plcnetapi_getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.LogicalNetwork)
    ))
  ,
  DESCRIPTOR = _GETNETWORKOVERVIEW,
  __module__ = 'devolo_idl_proto_plcnetapi_getnetworkoverview_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview)
  ))
_sym_db.RegisterMessage(GetNetworkOverview)
_sym_db.RegisterMessage(GetNetworkOverview.Device)
_sym_db.RegisterMessage(GetNetworkOverview.DataRate)
_sym_db.RegisterMessage(GetNetworkOverview.LogicalNetwork)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\006plcnetB\003net'))
# @@protoc_insertion_point(module_scope)



from copy import deepcopy
from unittest.mock import Mock

import pytest
from zeroconf import Zeroconf

try:
    from unittest.mock import AsyncMock
except ImportError:
    from asynctest import CoroutineMock as AsyncMock

from devolo_plc_api.device import Device

from ..mocks.mock_zeroconf import MockServiceBrowser, MockZeroconf


@pytest.fixture()
def mock_device(request):
    device = Device(ip=request.cls.ip)
    device._info = deepcopy(request.cls.device_info)
    device._loop = Mock()
    device._session = Mock()
    device._session.aclose = AsyncMock()
    device._zeroconf = Mock()
    device._zeroconf.close = lambda: None
    yield device


@pytest.fixture()
def mock_service_browser(mocker):
    mocker.patch("zeroconf.ServiceBrowser.__init__", MockServiceBrowser.__init__)
    mocker.patch("zeroconf.ServiceBrowser.cancel")


@pytest.fixture()
def mock_zeroconf(mocker):
    mocker.patch("zeroconf.Zeroconf.get_service_info", MockZeroconf.get_service_info)
    return Zeroconf()



import google.protobuf.json_format
import httpx
import pytest

from devolo_plc_api.device_api.devolo_idl_proto_deviceapi_ledsettings_pb2 import LedSettingsSetResponse
from devolo_plc_api.exceptions.device import DevicePasswordProtected, DeviceUnavailable


class TestProtobuf:

    def test_attribute_error(self, mock_protobuf):
        with pytest.raises(AttributeError):
            mock_protobuf.test()

    def test_url(self, request, mock_protobuf):
        ip = request.cls.ip
        path = request.cls.device_info["_dvl-plcnetapi._tcp.local."]["properties"]["Path"]
        version = request.cls.device_info["_dvl-plcnetapi._tcp.local."]["properties"]["Version"]
        assert mock_protobuf.url == f"http://{ip}:14791/{path}/{version}/"

    @pytest.mark.asyncio
    async def test__async_get(self, httpx_mock, mocker, mock_protobuf):
        httpx_mock.add_response()
        spy = mocker.spy(httpx.AsyncClient, "get")
        await mock_protobuf._async_get("LedSettingsGet")
        spy.assert_called_once()

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_wrong_password")
    async def test__async_get_wrong_password(self, mock_protobuf):
        with pytest.raises(DevicePasswordProtected):
            await mock_protobuf._async_get("LedSettingsGet")

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_device_unavailable")
    async def test__async_get_device_unavailable(self, mock_protobuf):
        with pytest.raises(DeviceUnavailable):
            await mock_protobuf._async_get("LedSettingsGet")

    def test__message_to_dict(self, mocker, mock_protobuf):
        spy = mocker.spy(google.protobuf.json_format._Printer, "_MessageToJsonObject")
        mock_protobuf._message_to_dict(LedSettingsSetResponse())
        spy.assert_called_once()

    @pytest.mark.asyncio
    async def test__async_post(self, httpx_mock, mocker, mock_protobuf):
        httpx_mock.add_response()
        spy = mocker.spy(httpx.AsyncClient, "post")
        await mock_protobuf._async_post("LedSettingsGet", "")
        spy.assert_called_once()

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_wrong_password")
    async def test__async_post_wrong_password(self, mock_protobuf):
        with pytest.raises(DevicePasswordProtected):
            await mock_protobuf._async_post("LedSettingsGet", "")

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_device_unavailable")
    async def test__async_post_device_unavailable(self, mock_protobuf):
        with pytest.raises(DeviceUnavailable):
            await mock_protobuf._async_post("LedSettingsGet", "")



import asyncio
from datetime import date
from unittest.mock import Mock, patch

import pytest
from zeroconf import ServiceBrowser, ServiceStateChange, Zeroconf

try:
    from unittest.mock import AsyncMock
except ImportError:
    from asynctest import CoroutineMock as AsyncMock

from devolo_plc_api.device import EMPTY_INFO
from devolo_plc_api.device_api.deviceapi import DeviceApi
from devolo_plc_api.exceptions.device import DeviceNotFound
from devolo_plc_api.plcnet_api.plcnetapi import PlcNetApi

from .mocks.mock_devices import state_change


class TestDevice:

    @pytest.mark.parametrize("feature", [""])
    def test_set_password(self, mock_device, device_api):
        mock_device.device = device_api
        mock_device.password = "super_secret"
        assert mock_device.device.password == "super_secret"

    @pytest.mark.asyncio
    async def test_async_connect(self, mocker, mock_device):
        with patch("devolo_plc_api.device.Device._get_device_info", new=AsyncMock()), \
             patch("devolo_plc_api.device.Device._get_plcnet_info", new=AsyncMock()):
            spy_device_info = mocker.spy(mock_device, "_get_device_info")
            spy_plcnet_info = mocker.spy(mock_device, "_get_plcnet_info")
            mock_device.device = object
            mock_device.plcnet = object
            await mock_device.async_connect()
            assert spy_device_info.call_count == 1
            assert spy_plcnet_info.call_count == 1
            assert mock_device._connected

    @pytest.mark.asyncio
    async def test_async_connect_not_found(self, mock_device):
        with patch("devolo_plc_api.device.Device._get_device_info", new=AsyncMock()), \
             patch("devolo_plc_api.device.Device._get_plcnet_info", new=AsyncMock()), \
             pytest.raises(DeviceNotFound):
            await mock_device.async_connect()
        assert not mock_device._connected

    def test_connect(self, mocker, mock_device):
        with patch("devolo_plc_api.device.Device.async_connect", new=AsyncMock()):
            spy_connect = mocker.spy(mock_device, "async_connect")
            mock_device.connect()
            assert spy_connect.call_count == 1

    @pytest.mark.asyncio
    async def test_async_disconnect(self, mocker, mock_device):
        spy_zeroconf = mocker.spy(mock_device._zeroconf, "close")
        spy_session = mocker.spy(mock_device._session, "aclose")
        await mock_device.async_disconnect()
        assert spy_zeroconf.call_count == 1
        assert spy_session.call_count == 1
        assert not mock_device._connected

    def test_disconnect(self, mocker, mock_device):
        with patch("devolo_plc_api.device.Device.async_disconnect", new=AsyncMock()):
            spy_connect = mocker.spy(mock_device, "async_disconnect")
            mock_device.disconnect()
            assert spy_connect.call_count == 1

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_device_api")
    async def test__get_device_info(self, mock_device):
        with patch("devolo_plc_api.device.Device._get_zeroconf_info", new=AsyncMock()):
            device_info = self.device_info['_dvl-deviceapi._tcp.local.']
            await mock_device._get_device_info()
            assert mock_device.firmware_date == date.fromisoformat(device_info["properties"]["FirmwareDate"])
            assert mock_device.firmware_version == device_info["properties"]["FirmwareVersion"]
            assert mock_device.serial_number == device_info["properties"]["SN"]
            assert mock_device.mt_number == device_info["properties"]["MT"]
            assert mock_device.product == device_info["properties"]["Product"]
            assert type(mock_device.device) == DeviceApi

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_device_api")
    async def test__get_device_info_timeout(self, mock_device):
        with patch("devolo_plc_api.device.Device._get_zeroconf_info", new=Mock()), \
             patch("asyncio.wait_for", new=AsyncMock(side_effect=asyncio.TimeoutError())):
            await mock_device._get_device_info()
            assert mock_device.device is None

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_plcnet_api")
    async def test__get_plcnet_info(self, mock_device):
        with patch("devolo_plc_api.device.Device._get_zeroconf_info", new=AsyncMock()):
            device_info = self.device_info["_dvl-plcnetapi._tcp.local."]
            await mock_device._get_plcnet_info()
            assert mock_device.mac == device_info["properties"]["PlcMacAddress"]
            assert mock_device.technology == device_info["properties"]["PlcTechnology"]
            assert type(mock_device.plcnet) == PlcNetApi

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_plcnet_api")
    async def test__get_plcnet_info_timeout(self, mock_device):
        with patch("devolo_plc_api.device.Device._get_zeroconf_info", new=Mock()), \
             patch("asyncio.wait_for", new=AsyncMock(side_effect=asyncio.TimeoutError())):
            await mock_device._get_plcnet_info()
            assert mock_device.plcnet is None

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_service_browser")
    async def test__get_zeroconf_info(self, mocker, mock_device):
        with patch("devolo_plc_api.device.Device._state_change", state_change):
            mock_device._info["_dvl-plcnetapi._tcp.local."] = EMPTY_INFO
            spy_cancel = mocker.spy(ServiceBrowser, "cancel")
            await mock_device._get_zeroconf_info("_dvl-plcnetapi._tcp.local.")
            assert spy_cancel.call_count == 1

    @pytest.mark.asyncio
    @pytest.mark.usefixtures("mock_service_browser")
    async def test__get_zeroconf_info_device_info_exists(self, mocker, mock_device):
        spy_cancel = mocker.spy(ServiceBrowser, "cancel")
        await mock_device._get_zeroconf_info("_dvl-plcnetapi._tcp.local.")
        assert spy_cancel.call_count == 0

    def test__state_change_no_service_info(self, mocker, mock_device):
        with patch("zeroconf.Zeroconf.get_service_info", return_value=None):
            service_type = "_dvl-plcnetapi._tcp.local."
            spy_service_info = mocker.spy(mock_device, "info_from_service")
            mock_device._state_change(Zeroconf(), service_type, service_type, ServiceStateChange.Added)
            assert spy_service_info.call_count == 0

    def test__state_change_added(self, mock_device, mock_zeroconf):
        service_type = "_dvl-plcnetapi._tcp.local."
        mock_device._state_change(mock_zeroconf, service_type, service_type, ServiceStateChange.Added)
        assert mock_device._info[service_type]["properties"]["new"] == "value"

    def test__state_change_removed(self, mock_device, mock_zeroconf):
        service_type = "_dvl-plcnetapi._tcp.local."
        mock_device._state_change(mock_zeroconf, service_type, service_type, ServiceStateChange.Removed)
        assert "new" not in mock_device._info[service_type]

    def test_info_from_service_no_address(self, mock_device):
        service_info = Mock()
        service_info.addresses = None
        assert mock_device.info_from_service(service_info) is None



try:
    from importlib.metadata import PackageNotFoundError, version
except ImportError:
    from importlib_metadata import PackageNotFoundError, version  # type: ignore[no-redef]

try:
    __version__ = version("devolo_plc_api")
except PackageNotFoundError:
    # package is not installed - e.g. pulled and run locally
    __version__ = "0.0.0"
